@@ -2,21 +2,24 @@ package Data::Util;
 
 use 5.008_001;
 use strict;
-use warnings;
+#use warnings;
 
-our $VERSION = '0.19_01';
+our $VERSION = '0.29_01';
 
 use Exporter qw(import);
 
-our $TESTING_PERL_ONLY;
-eval{
-	require XSLoader;
-	XSLoader::load(__PACKAGE__, $VERSION);
-} unless $TESTING_PERL_ONLY;
+{
+	local($!, $@);
+	our $TESTING_PERL_ONLY or eval{
+		require XSLoader;
+		XSLoader::load(__PACKAGE__, $VERSION);
+	};
+}
 
-eval q{require Data::Util::PurePerl} or die $@ # not to create "Data::Util::PurePerl" namespace
+require q{Data/Util/PurePerl.pm} # not to create "Data::Util::PurePerl" namespace
 	unless defined &instance;
 
+
 our @EXPORT_OK = qw(
 	is_scalar_ref is_array_ref is_hash_ref is_code_ref is_glob_ref is_regex_ref
 	is_instance is_invocant
@@ -27,9 +30,15 @@ our @EXPORT_OK = qw(
 	anon_scalar neat
 
 	get_stash
+
 	install_subroutine
+	uninstall_subroutine
 	get_code_info
 
+	curry
+	wrap_subroutine
+	subroutine_modifier
+
 	mkopt
 	mkopt_hash
 );
@@ -57,13 +66,14 @@ Data::Util - A selection of utilities for data and data types
 
 =head1 VERSION
 
-This document describes Data::Util version 0.19_01
+This document describes Data::Util version 0.29_01
 
 =head1 SYNOPSIS
 
 	use Data::Util qw(:validate);
 
 	sub foo{
+		# they will die if invalid values are supplied
 		my $sref = scalar_ref(shift);
 		my $aref = array_ref(shift);
 		my $href = hash_ref(shift);
@@ -88,19 +98,25 @@ This document describes Data::Util version 0.19_01
 		# ...
 	}
 
-	# to generate an anonymous scalar reference
-	use Data::Util qw(anon_scalar)
+	# miscelaneous
+	use Data::Util qw(:all);
 
 	my $ref_to_undef = anon_scalar();
 	$x = anon_scalar($x); # OK
 
-	# miscelaneous
-	use Data::Util qw(get_stash install_subroutine get_code_info neat);
-
 	my $stash = get_stash('Foo');
-	install_subroutine('Foo', hello => sub{ "Hello, world!\n" });
+
+	install_subroutine('Foo',
+		hello  => sub{ "Hello!\n" },
+		goodby => sub{ "Goodby!\n" },
+	);
+
+	print Foo::hello(); # Hello!
+
 	my($pkg, $name) = get_code_info(\&Foo::hello); # => ('Foo', 'hello')
-	print Foo::hello(); # Hello, world!
+	my $fqn         = get_code_info(\&Foo::hello); # => 'Foo::bar'
+
+	uninstall_subroutine('Foo', qw(hello goodby));
 
 	print neat("Hello!\n"); # => "Hello!\n"
 	print neat(3.14);       # => 3.14
@@ -108,7 +124,11 @@ This document describes Data::Util version 0.19_01
 
 =head1 DESCRIPTION
 
-This module provides utility functions for data and data types.
+This module provides utility functions for data and data types,
+including functions for subroutines.
+
+The implementation of this module is both Pure Perl and XS, so if you have a C
+compiler, all the functions the module provides are really faster.
 
 =head1 INTERFACE
 
@@ -154,7 +174,7 @@ C<< Scalar::Util::blessed($value) && $value->isa($class) >>.
 
 =item is_invocant(value)
 
-For an invocant, i.e. a blessed reference or existent class name.
+For an invocant, i.e. a blessed reference or existent package name.
 
 If I<value> is a valid class name but does not exist, it will return false.
 
@@ -201,10 +221,10 @@ For an instance of I<class>.
 
 =item invocant(value)
 
-For an invocant, i.e. a blessed reference or existent class name.
+For an invocant, i.e. a blessed reference or existent package name.
 
 If I<value> is a valid class name and the class exists, then it returns
-the canonical class name, which is logically cleanuped. That is, it does
+the canonical class name, which is logically cleaned up. That is, it does
 C<< $value =~ s/^::(?:main::)*//; >> before returns it.
 
 NOTE:
@@ -239,6 +259,8 @@ Generates an anonymous scalar reference to I<value>.
 
 Returns a neat string that is suitable to display.
 
+This is a smart version of C<<do{ defined($value) ? qq{"$value"} : 'undef' }>>.
+
 =item get_stash(package)
 
 Returns the symbol table hash (also known as B<stash>) of I<package>
@@ -247,13 +269,13 @@ if the stash exists.
 It is similar to C<< do{ no strict 'refs'; \%{$package.'::'} } >>,
 but does B<not> create the stash if I<package> does not exist.
 
-=item install_subroutine(package, name => subr)
+=item install_subroutine(package, name => subr [, ...])
 
 Installs I<subr> into I<package> as I<name>.
 
 It is similar to
 C<< do{ no strict 'refs'; *{$package.'::'.$name} = \&subr; } >>.
-In addtion, if I<subr> is an anonymous subroutine, it is relocated into
+In addition, if I<subr> is an anonymous subroutine, it is relocated into
 I<package> as a named subroutine I<&package::name>.
 
 To re-install I<subr>, use C<< no warnings 'redefine' >> directive:
@@ -261,19 +283,82 @@ To re-install I<subr>, use C<< no warnings 'redefine' >> directive:
 	no warnings 'redefine';
 	install_subroutine($package, $name => $subr);
 
+=item uninstall_subroutine(package, names...)
+
+Uninstalls I<names> from I<package>.
+
+It is similar to C<Sub::Delete::delete_sub()>, but uninstall multiple
+subroutines at a time.
+
 =item get_code_info(subr)
 
 Returns a pair of elements, the package name and the subroutine name of I<subr>.
 
-This is the same function as C<Sub::Identify::get_code_info()>.
+It is similar to C<Sub::Identify::get_code_info()>, but it returns the full
+qualified name in scalar context.
+
+=item curry(subr, args and/or placeholders)
+
+Makes I<subr> curried and returns the curried subroutine.
+
+This is also considered as lightweight closures.
+
+See also L<Data::Util::Curry>.
+
+=item wrap_subroutine(subr, ...)
+
+Wraps I<subr> with subroutine modifiers and returns the wrapped subroutine.
+This is also considered as lightweight closures.
+
+I<subr> must be a code reference or callable object.
+
+Optional arguments:
+C<< before => [subroutine(s)] >> called before I<subr>.
+C<< around => [subroutine(s)] >> called around I<subr>.
+C<< after  => [subroutine(s)] >> called after  I<subr>.
+
+This is considered as a constructor of wrapped subroutines, and
+C<subroutine_modifier()> property accessors.
+
+=item subroutine_modifier(wrapped)
+
+Returns whether I<wrapped> is a wrapped subroutine.
+
+=item subroutine_modifier(wrapped, property)
+
+Gets I<property> from I<wrapped>.
+
+Valid properties are: C<before>, C<around>, C<after> and C<original>.
+
+=item subroutine_modifier(wrapped, modifier => [subroutine(s)])
+
+Adds subroutine I<modifier> to I<wrapped>.
+
+Valid modifiers are: C<before>, C<around>, C<after>.
 
 =item mkopt(input, moniker, require_unique, must_be)
 
-This is the same function as C<Data::OptList::mkopt()>.
+Produces an array of an array reference from I<input>.
+
+It is similar to C<Data::OptList::mkopt()>. In addition to it,
+I<must_be> can be a HASH reference with C<< name => type >> pairs.
+
+For example:
+
+	my $optlist = mkopt(['foo', bar => [42]], $moniker, $uniq, { bar => 'ARRAY' });
+	# $optlist == [[foo => undef], [bar => [42]]
 
 =item mkopt_hash(input, moniker, must_be)
 
-This is the same function as C<Data::OptList::mkopt_hash()>.
+Produces a hash reference from I<input>.
+
+It is similar to C<Data::OptList::mkopt_hash()>. In addition to it,
+I<must_be> can be a HASH reference with C<< name => tyupe >> pairs.
+
+For example:
+
+	my $optlist = mkopt(['foo', bar => [42]], $moniker, { bar => 'ARRAY' });
+	# $optlist == {foo => undef, bar => [42]}
 
 =back
 
@@ -289,12 +374,22 @@ Please report any bugs or feature requests to the author.
 
 =head1 SEE ALSO
 
-L<Params::Util>.
-
 L<Scalar::Util>.
 
+L<overload>.
+
+L<Params::Util>.
+
+L<Sub::Install>.
+
 L<Sub::Identify>.
 
+L<Sub::Delete>.
+
+L<Sub::Curry>.
+
+L<Class::Method::Modifiers>.
+
 L<Data::OptList>.
 
 =head1 AUTHOR
